package com.persagy.dmp.rwd.edit.service;

import com.persagy.common.criteria.CriteriaExpressionParser;
import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.web.MapResponse;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.rwd.edit.entity.QClassDef;
import com.persagy.dmp.rwd.edit.entity.QRwdeditVersion;
import com.persagy.dmp.rwd.edit.entity.RwdeditVersion;
import com.persagy.dmp.rwd.edit.enumeration.EnumVersionState;
import com.persagy.dmp.rwd.edit.repository.RwdeditVersionRepository;
import com.querydsl.jpa.impl.JPAQuery;
import com.querydsl.jpa.impl.JPAQueryFactory;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.mock.web.MockHttpServletResponse;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.io.Writer;
import java.util.*;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@ExtendWith(MockitoExtension.class)
class RwdeditVersionServiceTest {

    @Mock
    private JPAQueryFactory mockQueryFactory;
    @Mock
    private CriteriaUtils mockCriteriaUtils;
    @Mock
    private RwdeditVersionRepository mockRepository;
    @Mock
    private ClassDefChangeRecordService mockClassDefChangeRecordService;
    @Mock
    private FuncidDefChangeRecordService mockFuncidDefChangeRecordService;

    @InjectMocks
    private RwdeditVersionService rwdeditVersionServiceUnderTest;

    @Test
    void testQuery() {
        // Setup
        final JacksonCriteria criteria = JacksonCriteria.newInstance();
        when(mockCriteriaUtils.query(eq(new QRwdeditVersion("variable")), any(CriteriaExpressionParser.class), any(JacksonCriteria.class))).thenReturn(new PagedResponse<>());

        // Run the test
        final PagedResponse<RwdeditVersion> result = rwdeditVersionServiceUnderTest.query(criteria);

        // Verify the results
    }

    @Test
    void testCreate() {
        // Setup
        final RwdeditVersion param = new RwdeditVersion();
        param.setVersion("version");
        param.setCreateTime(new GregorianCalendar(2020, Calendar.JANUARY, 1).getTime());
        param.setCreateUser("createUser");
        param.setState(EnumVersionState.CANCEL);

        when(mockRepository.count()).thenReturn(0L);

        // Configure RwdeditVersionRepository.save(...).
        final RwdeditVersion rwdeditVersion = new RwdeditVersion();
        rwdeditVersion.setVersion("version");
        rwdeditVersion.setCreateTime(new GregorianCalendar(2020, Calendar.JANUARY, 1).getTime());
        rwdeditVersion.setCreateUser("createUser");
        rwdeditVersion.setState(EnumVersionState.CANCEL);
        when(mockRepository.save(any(RwdeditVersion.class))).thenReturn(rwdeditVersion);

        // Run the test
        final MapResponse result = rwdeditVersionServiceUnderTest.create(param);

        // Verify the results
    }

    @Test
    void testUpdate() {
        // Setup
        final RwdeditVersion param = new RwdeditVersion();
        param.setVersion("version");
        param.setCreateTime(new GregorianCalendar(2020, Calendar.JANUARY, 1).getTime());
        param.setCreateUser("createUser");
        param.setState(EnumVersionState.CANCEL);

        // Configure RwdeditVersionRepository.findById(...).
        final RwdeditVersion rwdeditVersion1 = new RwdeditVersion();
        rwdeditVersion1.setVersion("version");
        rwdeditVersion1.setCreateTime(new GregorianCalendar(2020, Calendar.JANUARY, 1).getTime());
        rwdeditVersion1.setCreateUser("createUser");
        rwdeditVersion1.setState(EnumVersionState.CANCEL);
        final Optional<RwdeditVersion> rwdeditVersion = Optional.of(rwdeditVersion1);
        when(mockRepository.findById("id")).thenReturn(rwdeditVersion);

        // Configure RwdeditVersionRepository.save(...).
        final RwdeditVersion rwdeditVersion2 = new RwdeditVersion();
        rwdeditVersion2.setVersion("version");
        rwdeditVersion2.setCreateTime(new GregorianCalendar(2020, Calendar.JANUARY, 1).getTime());
        rwdeditVersion2.setCreateUser("createUser");
        rwdeditVersion2.setState(EnumVersionState.CANCEL);
        when(mockRepository.save(any(RwdeditVersion.class))).thenReturn(rwdeditVersion2);

        // Run the test
        final MapResponse result = rwdeditVersionServiceUnderTest.update(param);

        // Verify the results
        verify(mockClassDefChangeRecordService).updateStateByVersion(EnumVersionState.CANCEL, "version");
        verify(mockFuncidDefChangeRecordService).updateStateByVersion(EnumVersionState.CANCEL, "version");
    }

    @Test
    void testUpdate_RwdeditVersionRepositoryFindByIdReturnsAbsent() {
        // Setup
        final RwdeditVersion param = new RwdeditVersion();
        param.setVersion("version");
        param.setCreateTime(new GregorianCalendar(2020, Calendar.JANUARY, 1).getTime());
        param.setCreateUser("createUser");
        param.setState(EnumVersionState.CANCEL);

        when(mockRepository.findById("id")).thenReturn(Optional.empty());

        // Configure RwdeditVersionRepository.save(...).
        final RwdeditVersion rwdeditVersion = new RwdeditVersion();
        rwdeditVersion.setVersion("version");
        rwdeditVersion.setCreateTime(new GregorianCalendar(2020, Calendar.JANUARY, 1).getTime());
        rwdeditVersion.setCreateUser("createUser");
        rwdeditVersion.setState(EnumVersionState.CANCEL);
        when(mockRepository.save(any(RwdeditVersion.class))).thenReturn(rwdeditVersion);

        // Run the test
        final MapResponse result = rwdeditVersionServiceUnderTest.update(param);

        // Verify the results
        verify(mockClassDefChangeRecordService).updateStateByVersion(EnumVersionState.CANCEL, "version");
        verify(mockFuncidDefChangeRecordService).updateStateByVersion(EnumVersionState.CANCEL, "version");
    }

    @Test
    void testDelete() {
        // Setup
        final RwdeditVersion param = new RwdeditVersion();
        param.setVersion("version");
        param.setCreateTime(new GregorianCalendar(2020, Calendar.JANUARY, 1).getTime());
        param.setCreateUser("createUser");
        param.setState(EnumVersionState.CANCEL);

        // Configure RwdeditVersionRepository.findById(...).
        final RwdeditVersion rwdeditVersion1 = new RwdeditVersion();
        rwdeditVersion1.setVersion("version");
        rwdeditVersion1.setCreateTime(new GregorianCalendar(2020, Calendar.JANUARY, 1).getTime());
        rwdeditVersion1.setCreateUser("createUser");
        rwdeditVersion1.setState(EnumVersionState.CANCEL);
        final Optional<RwdeditVersion> rwdeditVersion = Optional.of(rwdeditVersion1);
        when(mockRepository.findById("id")).thenReturn(rwdeditVersion);

        when(mockClassDefChangeRecordService.query(any(JacksonCriteria.class))).thenReturn(new PagedResponse<>());
        when(mockFuncidDefChangeRecordService.query(any(JacksonCriteria.class))).thenReturn(new PagedResponse<>());

        // Run the test
        final MapResponse result = rwdeditVersionServiceUnderTest.delete(param);

        // Verify the results
        verify(mockRepository).delete(any(RwdeditVersion.class));
    }

    @Test
    void testDelete_RwdeditVersionRepositoryFindByIdReturnsAbsent() {
        // Setup
        final RwdeditVersion param = new RwdeditVersion();
        param.setVersion("version");
        param.setCreateTime(new GregorianCalendar(2020, Calendar.JANUARY, 1).getTime());
        param.setCreateUser("createUser");
        param.setState(EnumVersionState.CANCEL);

        when(mockRepository.findById("id")).thenReturn(Optional.empty());
        when(mockClassDefChangeRecordService.query(any(JacksonCriteria.class))).thenReturn(new PagedResponse<>());
        when(mockFuncidDefChangeRecordService.query(any(JacksonCriteria.class))).thenReturn(new PagedResponse<>());

        // Run the test
        final MapResponse result = rwdeditVersionServiceUnderTest.delete(param);

        // Verify the results
        verify(mockRepository).delete(any(RwdeditVersion.class));
    }

    @Test
    void testDownload() {
        // Setup
        final HttpServletResponse resp = new MockHttpServletResponse();
        when(mockClassDefChangeRecordService.getSqlList("version", false)).thenReturn(Arrays.asList("value"));
        when(mockFuncidDefChangeRecordService.getSqlList("version", false)).thenReturn(Arrays.asList("value"));

        // Run the test
        rwdeditVersionServiceUnderTest.download(resp, "version", false);

        // Verify the results
    }

    @Test
    void testDownload_ClassDefChangeRecordServiceReturnsNoItems() {
        // Setup
        final HttpServletResponse resp = new MockHttpServletResponse();
        when(mockClassDefChangeRecordService.getSqlList("version", false)).thenReturn(Collections.emptyList());
        when(mockFuncidDefChangeRecordService.getSqlList("version", false)).thenReturn(Arrays.asList("value"));

        // Run the test
        rwdeditVersionServiceUnderTest.download(resp, "version", false);

        // Verify the results
    }

    @Test
    void testDownload_FuncidDefChangeRecordServiceReturnsNoItems() {
        // Setup
        final HttpServletResponse resp = new MockHttpServletResponse();
        when(mockClassDefChangeRecordService.getSqlList("version", false)).thenReturn(Arrays.asList("value"));
        when(mockFuncidDefChangeRecordService.getSqlList("version", false)).thenReturn(Collections.emptyList());

        // Run the test
        rwdeditVersionServiceUnderTest.download(resp, "version", false);

        // Verify the results
    }

    @Test
    void testDownloadTotal() {
        // Setup
        final PrintWriter out = new PrintWriter(new StringWriter(), false);
        when(mockQueryFactory.selectFrom(new QClassDef("variable"))).thenReturn(new JPAQuery<>());

        // Run the test
        rwdeditVersionServiceUnderTest.downloadTotal(out);

        // Verify the results
    }

    @Test
    void testDownloadTotal_BrokenOut() {
        // Setup
        final PrintWriter out = new PrintWriter(new Writer() {

            private final IOException exception = new IOException("Error");

            @Override
            public void write(final char[] cbuf, final int off, final int len) throws IOException {
                throw exception;
            }

            @Override
            public void flush() throws IOException {
                throw exception;
            }

            @Override
            public void close() throws IOException {
                throw exception;
            }
        }, false);
        when(mockQueryFactory.selectFrom(new QClassDef("variable"))).thenReturn(new JPAQuery<>());

        // Run the test
        rwdeditVersionServiceUnderTest.downloadTotal(out);

        // Verify the results
    }
}
